#coding: utf-8
u'''
Smoke-тесты, вызываемые стандартным механизмом Django.
'''

import os
import sys

from django.db import transaction, connections, DEFAULT_DB_ALIAS
from django.conf import settings
from django.test import TransactionTestCase
from django.test.testcases import disable_transaction_methods, restore_transaction_methods

from .senders import DjangoClientSender
from .loaders import DebuggieLoader

class SmokeTestCase(TransactionTestCase):
    u'''

    .. method:: SmokeTestCase.test_if_there_is_a_smoke(self)

    Основной механизм обнаружения файлов с сохраненными запросами и
    обработки их. Срабатывает при запуске стандартными механизмом
    тестов Django и последовательно отсылает каждый из них, с проверкой
    результата.

    Если код ответа хотя бы одного из запросов будет отличаться от сохраненного,
    то по завершению отсылки всего массива запросов будет выброшено исключение.

    '''

    def __init__(self, *args, **kwargs):
        if getattr(self, 'multi_db', False):
            self.databases = connections
        else:
            self.databases = [DEFAULT_DB_ALIAS]
        super(SmokeTestCase, self).__init__(*args, **kwargs)


    def _fixture_setup(self):
        u'''
        Здесь мы вынужденно перекрываем метод загрузки данных
        из фикстур и дублируем часть его кода, т.к. monkey
        patching модуля транзакций зачем-то разместили здесь (sic!).
        '''

        for db in self.databases:
            transaction.enter_transaction_management(using=db)
            transaction.managed(True, using=db)
        disable_transaction_methods()

    def _fixture_teardown(self):
        u'''
        Здесь мы вынужденно перекрываем метод снесения данных
        из фикстур и дублируем часть его кода, т.к. восстановление
        методов транзакций зачем-то было помещено в него (sic!).
        '''
        restore_transaction_methods()
        for db in self.databases:
            transaction.rollback(using=db)
            transaction.leave_transaction_management(using=db)

    def test_if_there_is_a_smoke(self):        
        file_types = [('debuggie', '.ddump.txt'),]
        loader_mapping = {
            'debuggie': DebuggieLoader,
        }
        default_path = os.path.join(settings.PROJECT_ROOT, 'smoketrails', 'debuggie')
        errors = []
        storage_paths = getattr(settings, 'WELLSMOKED_PATHS', {'debuggie': default_path})

        for (ft, fn_ext) in file_types:
            path = storage_paths[ft]

            for root, dirs, files in os.walk(path):
                for fn in files:
                    if not fn.endswith(fn_ext):
                        continue

                    savepoint = transaction.savepoint()

                    loader = loader_mapping[ft]()
                    sender = DjangoClientSender()
                    with open(os.path.join(root, fn), 'r') as fp:
                        reqs = loader.load_requests(fp)

                        for req in reqs:
                            try:
                                resp_code, content = sender.send_request(req)
                                if resp_code != 200:
                                    errors.append((fn, resp_code, '{0} {1}'.format(req.verb, req.uri)))
                                    break
                            except:
                                errors.append((fn, 500, '{0} {1}'.format(req.verb, req.uri)))
                                break

                    transaction.savepoint_rollback(savepoint)

        if errors:
            for (fn, code, failing_uri) in errors:
                sys.stdout.write(u'==> {0}\n\t{1} {2}\n'.format(fn, code, failing_uri))
            raise AssertionError('{0} requests failed!'.format(len(errors)))
